@@ -119,8 +119,17 @@ struct _GstRTSPClientClass {
 
   void     (*tunnel_http_response)    (GstRTSPClient * client, GstRTSPMessage * request,
                                        GstRTSPMessage * response);
+  void     (*send_message)            (GstRTSPClient * client, GstRTSPContext *ctx,
+                                       GstRTSPMessage * response);
+
+  gboolean (*handle_sdp)              (GstRTSPClient *client, GstRTSPContext *ctx, GstRTSPMedia *media, GstSDPMessage *sdp);
+
+  void     (*announce_request)        (GstRTSPClient *client, GstRTSPContext *ctx);
+  void     (*record_request)          (GstRTSPClient *client, GstRTSPContext *ctx);
+  gchar*   (*check_requirements)      (GstRTSPClient *client, GstRTSPContext *ctx, gchar ** arr);
+
   /*< private >*/
-  gpointer _gst_reserved[GST_PADDING_LARGE-1];
+  gpointer _gst_reserved[GST_PADDING_LARGE-6];
 };
 
 GType                 gst_rtsp_client_get_type          (void);
