@@ -57,8 +57,9 @@ struct _GstRTSPSessionPrivate
 
   guint timeout;
   gboolean timeout_always_visible;
-  GTimeVal create_time;         /* immutable */
-  GTimeVal last_access;
+  GMutex last_access_lock;
+  gint64 last_access_monotonic_time;
+  gint64 last_access_real_time;
   gint expire_count;
 
   GList *medias;
@@ -132,8 +133,9 @@ gst_rtsp_session_init (GstRTSPSession * session)
   GST_INFO ("init session %p", session);
 
   g_mutex_init (&priv->lock);
+  g_mutex_init (&priv->last_access_lock);
   priv->timeout = DEFAULT_TIMEOUT;
-  g_get_current_time (&priv->create_time);
+
   gst_rtsp_session_touch (session);
 }
 
@@ -153,6 +155,7 @@ gst_rtsp_session_finalize (GObject * obj)
 
   /* free session id */
   g_free (priv->sessionid);
+  g_mutex_clear (&priv->last_access_lock);
   g_mutex_clear (&priv->lock);
 
   G_OBJECT_CLASS (gst_rtsp_session_parent_class)->finalize (obj);
@@ -555,9 +558,10 @@ gst_rtsp_session_touch (GstRTSPSession * session)
 
   priv = session->priv;
 
-  g_mutex_lock (&priv->lock);
-  g_get_current_time (&priv->last_access);
-  g_mutex_unlock (&priv->lock);
+  g_mutex_lock (&priv->last_access_lock);
+  priv->last_access_monotonic_time = g_get_monotonic_time ();
+  priv->last_access_real_time = g_get_real_time ();
+  g_mutex_unlock (&priv->last_access_lock);
 }
 
 /**
@@ -587,6 +591,52 @@ gst_rtsp_session_allow_expire (GstRTSPSession * session)
   g_atomic_int_add (&session->priv->expire_count, -1);
 }
 
+/**
+ * gst_rtsp_session_next_timeout_usec:
+ * @session: a #GstRTSPSession
+ * @now: (transfer none): the current monotonic time
+ *
+ * Get the amount of milliseconds till the session will expire.
+ *
+ * Returns: the amount of milliseconds since the session will time out.
+ */
+gint
+gst_rtsp_session_next_timeout_usec (GstRTSPSession * session, gint64 now)
+{
+  GstRTSPSessionPrivate *priv;
+  gint res;
+  GstClockTime last_access, now_ns;
+
+  g_return_val_if_fail (GST_IS_RTSP_SESSION (session), -1);
+
+  priv = session->priv;
+
+  g_mutex_lock (&priv->last_access_lock);
+  if (g_atomic_int_get (&priv->expire_count) != 0) {
+    /* touch session when the expire count is not 0 */
+    priv->last_access_monotonic_time = g_get_monotonic_time ();
+    priv->last_access_real_time = g_get_real_time ();
+  }
+
+  last_access = GST_USECOND * (priv->last_access_monotonic_time);
+
+  /* add timeout allow for 5 seconds of extra time */
+  last_access += priv->timeout * GST_SECOND + (5 * GST_SECOND);
+  g_mutex_unlock (&priv->last_access_lock);
+
+  now_ns = GST_USECOND * now;
+
+  if (last_access > now_ns) {
+    res = GST_TIME_AS_MSECONDS (last_access - now_ns);
+  } else {
+    res = 0;
+  }
+
+  return res;
+}
+
+/****** Deprecated API *******/
+
 /**
  * gst_rtsp_session_next_timeout:
  * @session: a #GstRTSPSession
@@ -595,7 +645,11 @@ gst_rtsp_session_allow_expire (GstRTSPSession * session)
  * Get the amount of milliseconds till the session will expire.
  *
  * Returns: the amount of milliseconds since the session will time out.
+ *
+ * Deprecated: Use gst_rtsp_session_next_timeout_usec() instead.
  */
+#ifndef GST_REMOVE_DEPRECATED
+#ifndef GST_DISABLE_DEPRECATED
 gint
 gst_rtsp_session_next_timeout (GstRTSPSession * session, GTimeVal * now)
 {
@@ -608,26 +662,53 @@ gst_rtsp_session_next_timeout (GstRTSPSession * session, GTimeVal * now)
 
   priv = session->priv;
 
-  g_mutex_lock (&priv->lock);
+  g_mutex_lock (&priv->last_access_lock);
   if (g_atomic_int_get (&priv->expire_count) != 0) {
     /* touch session when the expire count is not 0 */
-    g_get_current_time (&priv->last_access);
+    priv->last_access_monotonic_time = g_get_monotonic_time ();
+    priv->last_access_real_time = g_get_real_time ();
   }
 
-  last_access = GST_TIMEVAL_TO_TIME (priv->last_access);
+  last_access = GST_USECOND * (priv->last_access_real_time);
+
   /* add timeout allow for 5 seconds of extra time */
   last_access += priv->timeout * GST_SECOND + (5 * GST_SECOND);
-  g_mutex_unlock (&priv->lock);
+  g_mutex_unlock (&priv->last_access_lock);
 
   now_ns = GST_TIMEVAL_TO_TIME (*now);
 
-  if (last_access > now_ns)
+  if (last_access > now_ns) {
     res = GST_TIME_AS_MSECONDS (last_access - now_ns);
-  else
+  } else {
     res = 0;
+  }
 
   return res;
 }
+#endif
+#endif
+
+/**
+ * gst_rtsp_session_is_expired_usec:
+ * @session: a #GstRTSPSession
+ * @now: (transfer none): the current monotonic time
+ *
+ * Check if @session timeout out.
+ *
+ * Returns: %TRUE if @session timed out
+ */
+gboolean
+gst_rtsp_session_is_expired_usec (GstRTSPSession * session, gint64 now)
+{
+  gboolean res;
+
+  res = (gst_rtsp_session_next_timeout_usec (session, now) == 0);
+
+  return res;
+}
+
+
+/****** Deprecated API *******/
 
 /**
  * gst_rtsp_session_is_expired:
@@ -637,7 +718,11 @@ gst_rtsp_session_next_timeout (GstRTSPSession * session, GTimeVal * now)
  * Check if @session timeout out.
  *
  * Returns: %TRUE if @session timed out
+ *
+ * Deprecated: Use gst_rtsp_session_is_expired_usec() instead.
  */
+#ifndef GST_REMOVE_DEPRECATED
+#ifndef GST_DISABLE_DEPRECATED
 gboolean
 gst_rtsp_session_is_expired (GstRTSPSession * session, GTimeVal * now)
 {
@@ -647,3 +732,5 @@ gst_rtsp_session_is_expired (GstRTSPSession * session, GTimeVal * now)
 
   return res;
 }
+#endif
+#endif
